/*
 **************************************************************************
 * Copyright (c) 2014-2017, The Linux Foundation. All rights reserved.
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all copies.
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 **************************************************************************
 */

/*
 * @file nss_virt_if.h
 *	NSS Virtual interface message Structure and APIs
 */

#ifndef __NSS_VIRT_IF_H
#define __NSS_VIRT_IF_H

#include "nss_if.h"

/**
 * @addtogroup nss_virtual_if_subsystem
 * @{
 */

/**
 * nss_virt_if_msg_types
 *	Message types for virtual interface requests and responses.
 */
enum nss_virt_if_msg_types {
	NSS_VIRT_IF_OPEN = NSS_IF_OPEN,
	NSS_VIRT_IF_CLOSE = NSS_IF_CLOSE,
	NSS_VIRT_IF_LINK_STATE_NOTIFY = NSS_IF_LINK_STATE_NOTIFY,
	NSS_VIRT_IF_MTU_CHANGE = NSS_IF_MTU_CHANGE,
	NSS_VIRT_IF_MAC_ADDR_SET = NSS_IF_MAC_ADDR_SET,
	NSS_VIRT_IF_STATS_SYNC = NSS_IF_STATS,
	NSS_VIRT_IF_ISHAPER_ASSIGN = NSS_IF_ISHAPER_ASSIGN,
	NSS_VIRT_IF_BSHAPER_ASSIGN = NSS_IF_BSHAPER_ASSIGN,
	NSS_VIRT_IF_ISHAPER_UNASSIGN = NSS_IF_ISHAPER_UNASSIGN,
	NSS_VIRT_IF_BSHAPER_UNASSIGN = NSS_IF_BSHAPER_UNASSIGN,
	NSS_VIRT_IF_ISHAPER_CONFIG = NSS_IF_ISHAPER_CONFIG,
	NSS_VIRT_IF_BSHAPER_CONFIG = NSS_IF_BSHAPER_CONFIG,
	NSS_VIRT_IF_VSI_ASSIGN = NSS_IF_VSI_ASSIGN,
	NSS_VIRT_IF_VSI_UNASSIGN = NSS_IF_VSI_UNASSIGN,
	NSS_VIRT_IF_TX_CREATE_MSG = NSS_IF_MAX_MSG_TYPES + 1,
	NSS_VIRT_IF_TX_DESTROY_MSG,
	NSS_VIRT_IF_STATS_SYNC_MSG,
	NSS_VIRT_IF_MAX_MSG_TYPES,
};

/**
 * nss_virt_if_error_types
 *	Error types for the virtual interface.
 */
enum nss_virt_if_error_types {
	NSS_VIRT_IF_SUCCESS,
	NSS_VIRT_IF_CORE_FAILURE,
	NSS_VIRT_IF_ALLOC_FAILURE,
	NSS_VIRT_IF_DYNAMIC_IF_FAILURE,
	NSS_VIRT_IF_MSG_TX_FAILURE,
	NSS_VIRT_IF_REG_FAILURE,
	NSS_VIRT_IF_CORE_NOT_INITIALIZED,
};

/**
 * nss_virt_if_stats
 *	Virtual interface statistics received from the NSS.
 */
struct nss_virt_if_stats {
	struct nss_if_stats node_stats;	/**< Common statistics. */
	uint32_t tx_enqueue_failed;	/**< Tx enqueue failures in the firmware. */
	uint32_t shaper_enqueue_failed;	/**< Shaper enqueue failures in the firmware. */
};

/**
 * nss_virt_if_create_msg
 *	Configuration information for the virtual interface.
 */
struct nss_virt_if_create_msg {
	uint32_t flags;			/**< Interface flags. */
	uint8_t mac_addr[ETH_ALEN];	/**< MAC address. */
};

/**
 * nss_virt_if_destroy_msg
 *	Deletion information for the virtual interface.
 */
struct nss_virt_if_destroy_msg {
	int32_t reserved;		/**< Reserved for future use. */
};

/**
 * nss_virt_if_msg
 *	Data for sending and receiving virtual interface messages.
 */
struct nss_virt_if_msg {
	struct nss_cmn_msg cm;		/**< Common message header. */

	/**
	 * Payload of a virtual interface message.
	 */
	union {
		union nss_if_msgs if_msgs;
				/**< NSS interface base message. */
		struct nss_virt_if_create_msg if_create;
				/**< Rule for creating a virtual interface. */
		struct nss_virt_if_destroy_msg if_destroy;
				/**< Rule for destroying a virtual interface. */
		struct nss_virt_if_stats stats;
				/**< Virtual interface statistics. */
	} msg;			/**< Message payload. */
};

/*
 * nss_virt_if_pvt
 *	Private data information for the virtual interface.
 */
struct nss_virt_if_pvt {
	struct semaphore sem;
			/**< Semaphore to ensure that only one instance of a message is sent to the NSS. */
	struct completion complete;
			/**< Waits for message completion or time out. */
	int response;		/**< Message process response from the NSS firmware. */
	int sem_init_done;	/**< Semaphore initialization is done. */
};

/**
 * Callback function for virtual interface data.
 *
 * @datatypes
 * net_device \n
 * sk_buff \n
 * napi_struct
 *
 * @param[in] netdev  Pointer to the associated network device.
 * @param[in] skb     Pointer to the data socket buffer.
 * @param[in] napi    Pointer to the NAPI structure.
 */
typedef void (*nss_virt_if_data_callback_t)(struct net_device *netdev, struct sk_buff *skb, struct napi_struct *napi);

/**
 * Callback function for virtual interface messages.
 *
 * @datatypes
 * nss_cmn_msg
 *
 * @param[in] app_data  Pointer to the application context of the message.
 * @param[in] msg       Pointer to the message data.
 */
typedef void (*nss_virt_if_msg_callback_t)(void *app_data, struct nss_cmn_msg *msg);

/**
 * nss_virt_if_handle
 *	Context information for WLAN-to-NSS communication.
 */
struct nss_virt_if_handle {
	struct nss_ctx_instance *nss_ctx;	/**< NSS context. */
	int32_t if_num;				/**< Interface number. */
	struct net_device *ndev;		/**< Associated network device. */
	struct nss_virt_if_pvt *pvt;		/**< Private data structure. */
	struct nss_virt_if_stats stats;		/**< Virtual interface statistics. */
	atomic_t refcnt;			/**< Reference count. */
	nss_virt_if_msg_callback_t cb;		/**< Message callback. */
	void *app_data;		/**< Application data to be passed to the callback. */
};

/**
 * nss_virt_if_create
 *	Creates a virtual interface asynchronously.
 *
 * @datatypes
 * net_device \n
 * nss_virt_if_msg_callback_t
 *
 * @param[in] netdev    Pointer to the associated network device.
 * @param[in] cb        Callback function for the message. This callback is
                        invoked when the response from the firmware is received.
 * @param[in] app_data  Pointer to the application context of the message.
 *
 * @return
 * Status of the Tx operation.
 */
extern int nss_virt_if_create(struct net_device *netdev, nss_virt_if_msg_callback_t cb, void *app_data);

/**
 * nss_virt_if_create_sync
 *	Creates a virtual interface synchronously.
 *
 * @datatypes
 * net_device
 *
 * @param[in] netdev  Pointer to the associated network device.
 *
 * @return
 * Pointer to nss_virt_if_handle.
 */
extern struct nss_virt_if_handle *nss_virt_if_create_sync(struct net_device *netdev);

/**
 * nss_virt_if_destroy
 *	Destroys the virtual interface asynchronously.
 *
 * @datatypes
 * nss_virt_if_handle \n
 * nss_virt_if_msg_callback_t
 *
 * @param[in,out] handle    Pointer to the virtual interface handle (provided during
 *                          dynamic interface allocation).
 * @param[in]     cb        Callback function for the message. This callback is
 *                          invoked when the response from the firmware is received.
 * @param[in]     app_data  Pointer to the application context of the message.
 *
 * @return
 * Status of the Tx operation.
 *
 * @dependencies
 * The interface must have been previously created.
 */
extern nss_tx_status_t nss_virt_if_destroy(struct nss_virt_if_handle *handle, nss_virt_if_msg_callback_t cb, void *app_data);

/**
 * nss_virt_if_destroy_sync
 *	Destroys the virtual interface synchronously.
 *
 * @datatypes
 * nss_virt_if_handle
 *
 * @param[in,out] handle  Pointer to the virtual interface handle (provided during
 *                        dynamic interface allocation).
 *
 * @return
 * Status of the Tx operation.
 *
 * @dependencies
 * The interface must have been previously created.
 */
extern nss_tx_status_t nss_virt_if_destroy_sync(struct nss_virt_if_handle *handle);

/**
 * nss_virt_if_tx_msg
 *	Sends a message to the virtual interface.
 *
 * @datatypes
 * nss_ctx_instance \n
 * nss_virt_if_msg
 *
 * @param[in] nss_ctx  Pointer to the NSS context (provided during registration).
 * @param[in] nvim     Pointer to the virtual interface message.
 *
 * @return
 * command Tx status
 */
extern nss_tx_status_t nss_virt_if_tx_msg(struct nss_ctx_instance *nss_ctx, struct nss_virt_if_msg *nvim);

/**
 * nss_virt_if_tx_buf
 *	Forwards virtual interface packets to the NSS.
 *
 * @datatypes
 * nss_virt_if_handle \n
 * sk_buff
 *
 * @param[in,out] handle  Pointer to the virtual interface handle (provided during
 *                        registration).
 * @param[in]    skb     Pointer to the data socket buffer.
 *
 * @return
 * Status of the Tx operation.
 */
extern nss_tx_status_t nss_virt_if_tx_buf(struct nss_virt_if_handle *handle,
						struct sk_buff *skb);

/**
 * nss_virt_if_register
 *	Registers a virtual Interface with NSS driver,
 *
 * @datatypes
 * nss_virt_if_handle \n
 * nss_virt_if_data_callback_t \n
 * net_device
 *
 * @param[in,out] handle        Pointer to the virtual interface handle(provided during
 *                              dynamic interface allocation).
 * @param[in]    data_callback  Callback handler for virtual data packets
 * @param[in]    netdev         Pointer to the associated network device.
 *
 * @return
 * Status of the Tx operation.
 */
extern void nss_virt_if_register(struct nss_virt_if_handle *handle,
					nss_virt_if_data_callback_t data_callback,
					struct net_device *netdev);

/**
 * nss_virt_if_unregister
 *	Deregisters a virtual interface from the NSS driver.
 *
 * @datatypes
 * nss_virt_if_handle
 *
 * @param[in,out] handle  Pointer to the virtual interface handle.
 *
 * @return
 * None.
 */
extern void nss_virt_if_unregister(struct nss_virt_if_handle *handle);

/**
 * nss_virt_if_copy_stats
 *	Gets statistics for the virtual interface from the NSS driver.
 *
 * @param[in]  if_num  NSS interface number (provided during dynamic interface allocation).
 * @param[in]  i       Index of statistics.
 * @param[out] line    Buffer into which the statistics are copied.
 *
 * @return
 * Returns 0 if if_num is not in range, or the number of bytes copied.
 */
extern int32_t nss_virt_if_copy_stats(int32_t if_num, int i, char *line);

/**
 * nss_virt_if_get_interface_num
 *	Returns the virtual interface number associated with the handle.
 *
 * @datatypes
 * nss_virt_if_handle
 *
 * @param[in] handle  Pointer to the virtual interface handle(provided during
                      dynamic interface allocation).
 *
 * @return
 * virtual interface number.
 */
extern int32_t nss_virt_if_get_interface_num(struct nss_virt_if_handle *handle);

/**
 * @}
 */

#endif /* __NSS_VIRT_IF_H */
